#include <list>
#include <vector>
#include <string>
#include <algorithm>
#include <fstream.h>
#include "incop.h"
#include "csproblem.h"
#include "maxclique.h"
#include "color.h"
#include "move.h"

extern ofstream* ofile;  // le fichier de sortie
/** Tout ce qui concerne le probleme max-clique : 
on essaie de trouver une clique de taille n donne en minimisant les conflits (aretes manquantes
dans l'ensemble des n sommets candidats
Un mouvement est un change entre un sommet dans la clique candidate participant  un conflit 
et un sommet hors clique (remainvariables)
Utilisation de ExchangeMove dfini  dans move  
*/

/* meme format de fichier DIMACS que pour le coloriage  */
void lire_debut_fichier_clique(ifstream & file, int& nbvar, int& nbconst)
{lire_debut_fichier_coloriage (file,  nbvar,  nbconst);}

void lire_fichier_clique (ifstream& file, vector<int>* connexions,int ** constraint1)
{lire_fichier_coloriage (file,  connexions, constraint1);}


CliqueProblem* clique_problem_creation (int clsize,ifstream & file)
{

  // Declaration des variables contenant les structures de donnes des problemes
  int **constraint1; // utilise dans tout csp binaire
  int nbvar; int nbconst;
  lire_debut_fichier_clique(file,nbvar,nbconst); // pour dimensionner les structures
  
  constraint1 =  csp_constraintdatastructure(nbvar);

  int* dom = new int[nbvar];
  vector<int>* tabdom = new vector<int> [nbvar] ; // les differents types de domaines 
  // Initialisation des structures de donnes des problmes
  vector<int>* connex = new vector<int> [nbvar];

  lire_fichier_clique(file,connex,constraint1);
  CliqueProblem * problem = probleme_maxclique(nbvar,nbconst,clsize,constraint1);
  // mise en place des domaines et connexions
  problem->set_domains_connections(dom,tabdom,connex);
  problem->init_domain_tabdomain();
  return problem;
}


CliqueProblem::CliqueProblem (int nvar, int nconst,int clsize) :
BinaryCSProblem (nvar,nconst){domainsize=2;cliquesize=clsize;}

CliqueProblem* probleme_maxclique(int nbvar,int nbconst,int clsize,int** constraint1)
{CliqueProblem* p1 = new CliqueProblem(nbvar,nbconst,clsize);
p1->constraints=constraint1; 
return p1;
}

int* CliqueProblem::clique(Configuration* configuration)
{return ((CliqueConfiguration*)configuration)->clique;}

int* CliqueProblem::remainvariables(Configuration* configuration)
{return ((CliqueConfiguration*)configuration)->remainvariables;}


int CliqueProblem::config_evaluation (Configuration* configuration)
{int valeur=0;
 configuration->init_conflicts();
 for(int i=0; i<nbvar; i++)
   {int cliquemember=0;
   for(int j=0; j< cliquesize; j++)
    {if (clique(configuration)[j]==i)
      {cliquemember=1; break;}
    }
   for(int j=0; j< cliquesize; j++)
     {if ((i <clique(configuration)[j]) &&
	  constraints[i][clique(configuration)[j]]==0 ||
	  (!cliquemember && i > clique(configuration)[j] && constraints[clique(configuration)[j]][i]==0))
       {if (cliquemember) {valeur++; }
       configuration->incr_conflicts(i,1,1,1);
       if (cliquemember) configuration->incr_conflicts(clique(configuration)[j],1,1,1);}
     }
   }

 
 return valeur;
}

int CliqueProblem::config_evaluation_verif (Configuration* configuration)
{int valeur=0;
 for(int i=0; i<nbvar; i++)
   {int cliquemember=0;
   for(int j=0; j< cliquesize; j++)
    {if (clique(configuration)[j]==i)
      {cliquemember=1; break;}
    }
   for(int j=0; j< cliquesize; j++)
     {if ((i <clique(configuration)[j]) &&
	  constraints[i][clique(configuration)[j]]==0 ||
	  (!cliquemember && i > clique(configuration)[j] && constraints[i][clique(configuration)[j]]==0))
       {if (cliquemember) {valeur++; }
       }
     }
   }
 /*

  for (int i =0;i<nbvar;i++)
    *ofile << " nbconf " << i << " " << ((FullincrCSPConfiguration*)configuration)->tabconflicts[i][1] << endl; 
    */
  return valeur;
}



void CliqueProblem::fullincr_update_conflicts(FullincrCSPConfiguration* configuration,Move* move)
{int var_out = clique(configuration)[((ExchangeMove*)move)-> variable1];
   int var_in = remainvariables(configuration)[((ExchangeMove*)move)-> variable2];

  for (int j=0; j< nbvar; j++)
    {if (((j > var_out) && constraints[var_out][j]==0) ||
	 ((j < var_out) && constraints[j][var_out]==0))
      {
	configuration->tabconflicts[j][1]--;
	//		*ofile << " var_out " << var_out <<  "j- " << j << " " <<configuration->tabconflicts[j][1] << endl; 
      }
    if (((j > var_in) && constraints[var_in][j]==0) ||
	((j < var_in) && constraints[j][var_in]==0))
      {
	configuration->tabconflicts[j][1]++;
	//	*ofile << "var_in" << var_in <<  "j+ " << j << " " <<configuration->tabconflicts[j][1] << endl; 
      }
    }

}




void CliqueProblem::move_execution(Configuration* configuration,Move* move)
{ OpProblem::move_execution(configuration,move);
  int var_out = clique(configuration)[((ExchangeMove*)move)-> variable1];
  int var_in = remainvariables(configuration)[((ExchangeMove*)move)-> variable2];
  configuration->config[var_in]=1;
  configuration->config[var_out]=0;
  ((CliqueConfiguration*)configuration)->clique[((ExchangeMove*)move)-> variable1]=var_in;
  ((CliqueConfiguration*)configuration)->remainvariables[((ExchangeMove*)move)-> variable2]=var_out;
  //   int value1 = config_evaluation_verif(configuration);
  //      if (value1 != move->valuation)
  //           *ofile << "erreur " << value1 << " " << move->valuation << endl;
}

/* non implant */
void CliqueProblem::incr_update_conflicts(IncrCSPConfiguration* configuration,Move* move)
{;}

/* pour les mouvements swap */
Move* CliqueProblem::create_move()
{ExchangeMove* move = new ExchangeMove();
 return (Move*)move;
}



void CliqueProblem::next_move
(Configuration* configuration, Move* move, NeighborhoodSearch* nbhs)
{int conflictvariable = 0; int index1;
  while(!conflictvariable)
    {index1 =  (int) (drand48() * cliquesize);
    if (((FullincrCSPConfiguration*)configuration)->tabconflicts[clique(configuration)[index1]][1]) 
      conflictvariable=1;
    }
  ((ExchangeMove*)move)->variable1 = index1;
  if (nbhs->val_conflict)
    ((ExchangeMove*)move)->variable2 = minconflict_invariable(configuration);
  else
    ((ExchangeMove*)move)->variable2 = (int) (drand48 () * (nbvar - cliquesize));
  
  move->valuation = move_evaluation (configuration,move);
}

int CliqueProblem::minconflict_invariable(Configuration* configuration)
{ int minpromises=RAND_MAX; int promises=0; int k1=1; int j=0;
   for (int i =0; i< nbvar-cliquesize; i++)
     {
     promises= ((CliqueConfiguration*)configuration)->tabconflicts[remainvariables(configuration)[i]][1];
     if (promises < minpromises)
       {minpromises = promises; j=i; k1=1;}
     else if (promises == minpromises)
       {k1++;
       if (drand48() < 1/(double)k1) j=i;
       }
     }
   return j;
}


int CliqueProblem::move_evaluation (Configuration* configuration, Move* move)
{
  int var_out = clique(configuration)[((ExchangeMove*)move)-> variable1];
  int var_in = remainvariables(configuration)[((ExchangeMove*)move)-> variable2];
  int valeur = configuration->valuation +
  ((FullincrCSPConfiguration*)configuration)->tabconflicts[var_in][1]-
  ((FullincrCSPConfiguration*)configuration)->tabconflicts[var_out][1];
  if (((var_out > var_in) && constraints[var_in][var_out] ==0) ||
      ((var_in > var_out) && constraints[var_out][var_in] ==0))
    valeur--;
  //      *ofile << " conflits var in " << ((FullincrCSPConfiguration*)configuration)->tabconflicts[var_in][1] << " conflits var out " << ((FullincrCSPConfiguration*)configuration)->tabconflicts[var_out][1] << " valeur mvt " << valeur << endl;
  return valeur;
}

void CliqueProblem::random_configuration(Configuration* configuration)
{
  for (int j=0; j<cliquesize; j++)
    clique(configuration)[j]=nbvar;
  for (int i=0; i<nbvar; i++)
    configuration->config[i]=0;
  for(int j=0;j<cliquesize;j++)
    {  int var2 = (int) (drand48 () * (nbvar - j));
    int index=0;
    for (int k=0;k<nbvar;k++)
      {if (configuration->config[k]==0)
	index++;
      if (index > var2) {((CliqueConfiguration*)configuration)->clique[j]=k; configuration->config[k]=1; break;}
      }
    }
  int k=0;
  for(int i=0;i<nbvar;i++)
    if (configuration->config[i]==0)
      {((CliqueConfiguration*)configuration)->remainvariables[k]=i;k++;}

  //  *ofile << " clique courante " ;
  //  for (int j=0; j< cliquesize; j++)
  //    *ofile << clique(configuration)[j] << " " ;
  //  *ofile << endl;
}
  
void CliqueProblem::solution_write()
{ *ofile<< " meilleure clique" << endl;
for (int j=0; j< cliquesize; j++)
    *ofile << ((CliqueConfiguration*)best_config)->clique[j] << " " ;
  *ofile << endl;
}


Configuration* CliqueProblem::create_configuration ()
{return (new  CliqueConfiguration (nbvar,cliquesize));
}

CliqueConfiguration::CliqueConfiguration(int nbv, int cliques) : FullincrCSPConfiguration(nbv,2)
{cliquesize=cliques;
 clique = new int[cliquesize];
 remainvariables = new int[nbv-cliquesize];
}

CliqueConfiguration::~CliqueConfiguration()
{delete clique; delete remainvariables;}

void CliqueConfiguration::copy_element (Configuration* config2)
{FullincrCSPConfiguration::copy_element(config2);
for (int i=0; i<cliquesize ;i++)
  clique[i]=((CliqueConfiguration*)config2)->clique[i];
for (int i=0; i<nbvar- cliquesize ;i++)
  remainvariables[i]=((CliqueConfiguration*)config2)->remainvariables[i];
}
